@@ -49,15 +49,12 @@
 #include "CreateRequestCommand.h"
 #include "InitiateConnectionCommandFactory.h"
 #include "SleepCommand.h"
-#ifdef ENABLE_ASYNC_DNS
-#include "AsyncNameResolver.h"
-#endif // ENABLE_ASYNC_DNS
 #include "StreamCheckIntegrityEntry.h"
 #include "PieceStorage.h"
 #include "Socket.h"
 #include "message.h"
 #include "prefs.h"
-#include "StringFormat.h"
+#include "fmt.h"
 #include "ServerStat.h"
 #include "RequestGroupMan.h"
 #include "A2STR.h"
@@ -66,8 +63,12 @@
 #include "DownloadContext.h"
 #include "wallclock.h"
 #include "NameResolver.h"
-#include "ServerStatMan.h"
-#include "FileAllocationEntry.h"
+#include "uri.h"
+#include "FileEntry.h"
+#include "error_code.h"
+#ifdef ENABLE_ASYNC_DNS
+#include "AsyncNameResolver.h"
+#endif // ENABLE_ASYNC_DNS
 #ifdef ENABLE_MESSAGE_DIGEST
 # include "ChecksumCheckIntegrityEntry.h"
 #endif // ENABLE_MESSAGE_DIGEST
@@ -89,7 +90,7 @@ AbstractCommand::AbstractCommand(cuid_t cuid,
   nameResolverCheck_(false),
   incNumConnection_(incNumConnection)
 {
-  if(!socket_.isNull() && socket_->isOpen()) {
+  if(socket_ && socket_->isOpen()) {
     setReadCheckSocket(socket_);
   }
   if(incNumConnection_) {
@@ -113,58 +114,50 @@ AbstractCommand::~AbstractCommand() {
 }
 
 bool AbstractCommand::execute() {
-  if(getLogger()->debug()) {
-    getLogger()->debug("CUID#%s - socket: read:%d, write:%d, hup:%d, err:%d",
-                       util::itos(getCuid()).c_str(),
-                       readEventEnabled(),
-                       writeEventEnabled(),
-                       hupEventEnabled(),
-                       errorEventEnabled());
-  }
+  A2_LOG_DEBUG(fmt("CUID#%lld - socket: read:%d, write:%d, hup:%d, err:%d",
+                   getCuid(),
+                   readEventEnabled(),
+                   writeEventEnabled(),
+                   hupEventEnabled(),
+                   errorEventEnabled()));
   try {
     if(requestGroup_->downloadFinished() || requestGroup_->isHaltRequested()) {
       return true;
     }
-    if(!req_.isNull() && req_->removalRequested()) {
-      if(getLogger()->debug()) {
-        getLogger()->debug
-          ("CUID#%s - Discard original URI=%s because it is requested.",
-           util::itos(getCuid()).c_str(), req_->getUri().c_str());
-      }
+    if(req_ && req_->removalRequested()) {
+      A2_LOG_DEBUG(fmt("CUID#%lld - Discard original URI=%s because it is"
+                       " requested.",
+                       getCuid(), req_->getUri().c_str()));
       return prepareForRetry(0);
     }
-    if(!getPieceStorage().isNull()) {
+    if(getPieceStorage()) {
       segments_.clear();
       getSegmentMan()->getInFlightSegment(segments_, getCuid());
-      if(!req_.isNull() && segments_.empty()) {
+      if(req_ && segments_.empty()) {
         // This command previously has assigned segments, but it is
         // canceled. So discard current request chain.  Plus, if no
         // segment is available when http pipelining is used.
-        if(getLogger()->debug()) {
-          getLogger()->debug("CUID#%s - It seems previously assigned segments"
-                             " are canceled. Restart.",
-                             util::itos(getCuid()).c_str());
-        }
+        A2_LOG_DEBUG(fmt("CUID#%lld - It seems previously assigned segments"
+                         " are canceled. Restart.",
+                         getCuid()));
         // Request::isPipeliningEnabled() == true means aria2
         // accessed the remote server and discovered that the server
         // supports pipelining.
-        if(!req_.isNull() && req_->isPipeliningEnabled()) {
+        if(req_ && req_->isPipeliningEnabled()) {
           e_->poolSocket(req_, createProxyRequest(), socket_);
         }
         return prepareForRetry(0);
       }
       // TODO it is not needed to check other PeerStats every time.
       // Find faster Request when no segment is available.
-      if(!req_.isNull() && fileEntry_->countPooledRequest() > 0 &&
+      if(req_ && fileEntry_->countPooledRequest() > 0 &&
          !getPieceStorage()->hasMissingUnusedPiece()) {
         SharedHandle<Request> fasterRequest = fileEntry_->findFasterRequest(req_);
-        if(!fasterRequest.isNull()) {
-          if(getLogger()->info()) {
-            getLogger()->info("CUID#%s - Use faster Request hostname=%s, port=%u",
-                              util::itos(getCuid()).c_str(),
-                              fasterRequest->getHost().c_str(),
-                              fasterRequest->getPort());
-          }
+        if(fasterRequest) {
+          A2_LOG_INFO(fmt("CUID#%lld - Use faster Request hostname=%s, port=%u",
+                          getCuid(),
+                          fasterRequest->getHost().c_str(),
+                          fasterRequest->getPort()));
           // Cancel current Request object and use faster one.
           fileEntry_->removeRequest(req_);
           Command* command =
@@ -185,18 +178,18 @@ bool AbstractCommand::execute() {
        (!checkSocketIsReadable_ && !checkSocketIsWritable_ &&
         !nameResolverCheck_)) {
       checkPoint_ = global::wallclock;
-      if(!getPieceStorage().isNull()) {
-        if(req_.isNull() || req_->getMaxPipelinedRequest() == 1 ||
+      if(getPieceStorage()) {
+        if(!req_ || req_->getMaxPipelinedRequest() == 1 ||
            // Why the following condition is necessary? That's because
            // For single file download, SegmentMan::getSegment(cuid)
            // is more efficient.
            getDownloadContext()->getFileEntries().size() == 1) {
-          size_t maxSegments = req_.isNull()?1:req_->getMaxPipelinedRequest();
+          size_t maxSegments = req_?req_->getMaxPipelinedRequest():1;
           size_t minSplitSize = calculateMinSplitSize();
           while(segments_.size() < maxSegments) {
             SharedHandle<Segment> segment =
               getSegmentMan()->getSegment(getCuid(), minSplitSize);
-            if(segment.isNull()) {
+            if(!segment) {
               break;
             } else {
               segments_.push_back(segment);
@@ -206,16 +199,12 @@ bool AbstractCommand::execute() {
             // TODO socket could be pooled here if pipelining is
             // enabled...  Hmm, I don't think if pipelining is enabled
             // it does not go here.
-            if(getLogger()->info()) {
-              getLogger()->info(MSG_NO_SEGMENT_AVAILABLE,
-                                util::itos(getCuid()).c_str());
-            }
+            A2_LOG_INFO(fmt(MSG_NO_SEGMENT_AVAILABLE,
+                            getCuid()));
             // When all segments are ignored in SegmentMan, there are
             // no URIs available, so don't retry.
             if(getSegmentMan()->allSegmentsIgnored()) {
-              if(getLogger()->debug()) {
-                getLogger()->debug("All segments are ignored.");
-              }
+              A2_LOG_DEBUG("All segments are ignored.");
               return true;
             } else {
               return prepareForRetry(1);
@@ -236,37 +225,47 @@ bool AbstractCommand::execute() {
       }
       return executeInternal();
     } else if(errorEventEnabled()) {
-      throw DL_RETRY_EX
-        (StringFormat(MSG_NETWORK_PROBLEM,
-                      socket_->getSocketError().c_str()).str());
+      throw DL_RETRY_EX(fmt(MSG_NETWORK_PROBLEM,
+                            socket_->getSocketError().c_str()));
     } else {
       if(checkPoint_.difference(global::wallclock) >= timeout_) {
         // timeout triggers ServerStat error state.
-
         SharedHandle<ServerStat> ss =
           e_->getRequestGroupMan()->getOrCreateServerStat(req_->getHost(),
                                                           req_->getProtocol());
         ss->setError();
-
-        throw DL_RETRY_EX2(EX_TIME_OUT, downloadresultcode::TIME_OUT);
+        // Purging IP address cache to renew IP address.
+        A2_LOG_DEBUG(fmt("CUID#%lld - Marking IP address %s as bad",
+                         getCuid(),
+                         req_->getConnectedAddr().c_str()));
+        e_->markBadIPAddress(req_->getConnectedHostname(),
+                             req_->getConnectedAddr(),
+                             req_->getConnectedPort());
+        if(e_->findCachedIPAddress
+           (req_->getConnectedHostname(), req_->getConnectedPort()).empty()) {
+          A2_LOG_DEBUG(fmt("CUID#%lld - All IP addresses were marked bad."
+                           " Removing Entry.",
+                           getCuid()));
+          e_->removeCachedIPAddress
+            (req_->getConnectedHostname(), req_->getConnectedPort());
+        }
+        throw DL_RETRY_EX2(EX_TIME_OUT, error_code::TIME_OUT);
       }
       e_->addCommand(this);
       return false;
     }
   } catch(DlAbortEx& err) {
-    if(req_.isNull()) {
-      if(getLogger()->debug()) {
-        getLogger()->debug(EX_EXCEPTION_CAUGHT, err);
-      }
+    if(!req_) {
+      A2_LOG_DEBUG_EX(EX_EXCEPTION_CAUGHT, err);
     } else {
-      getLogger()->error
-        (MSG_DOWNLOAD_ABORTED,
-         DL_ABORT_EX2(StringFormat
-                      ("URI=%s", req_->getCurrentUri().c_str()).str(),err),
-         util::itos(getCuid()).c_str(), req_->getUri().c_str());
-      fileEntry_->addURIResult(req_->getUri(), err.getCode());
-      requestGroup_->setLastUriResult(req_->getUri(), err.getCode());
-      if(err.getCode() == downloadresultcode::CANNOT_RESUME) {
+      A2_LOG_ERROR_EX(fmt(MSG_DOWNLOAD_ABORTED,
+                          getCuid(),
+                          req_->getUri().c_str()),
+                      DL_ABORT_EX2(fmt("URI=%s", req_->getCurrentUri().c_str()),
+                                   err));
+      fileEntry_->addURIResult(req_->getUri(), err.getErrorCode());
+      requestGroup_->setLastErrorCode(err.getErrorCode());
+      if(err.getErrorCode() == error_code::CANNOT_RESUME) {
         requestGroup_->increaseResumeFailureCount();
       }
     }
@@ -274,30 +273,27 @@ bool AbstractCommand::execute() {
     tryReserved();
     return true;
   } catch(DlRetryEx& err) {
-    assert(!req_.isNull());
-    if(getLogger()->info()) {
-      getLogger()->info
-        (MSG_RESTARTING_DOWNLOAD,
-         DL_RETRY_EX2(StringFormat
-                      ("URI=%s", req_->getCurrentUri().c_str()).str(),
-                      err),
-         util::itos(getCuid()).c_str(), req_->getUri().c_str());
-    }
+    assert(req_);
+    A2_LOG_INFO_EX(fmt(MSG_RESTARTING_DOWNLOAD,
+                       getCuid(), req_->getUri().c_str()),
+                   DL_RETRY_EX2(fmt("URI=%s", req_->getCurrentUri().c_str()),
+                                err));
     req_->addTryCount();
     req_->resetRedirectCount();
+    req_->resetUri();
     const unsigned int maxTries = getOption()->getAsInt(PREF_MAX_TRIES);
     bool isAbort = maxTries != 0 && req_->getTryCount() >= maxTries;
     if(isAbort) {
-      if(getLogger()->info()) {
-        getLogger()->info(MSG_MAX_TRY,
-                          util::itos(getCuid()).c_str(), req_->getTryCount());
-      }
-      getLogger()->error(MSG_DOWNLOAD_ABORTED, err,
-                         util::itos(getCuid()).c_str(),
-                         req_->getUri().c_str());
-      fileEntry_->addURIResult(req_->getUri(), err.getCode());
-      requestGroup_->setLastUriResult(req_->getUri(), err.getCode());
-      if(err.getCode() == downloadresultcode::CANNOT_RESUME) {
+      A2_LOG_INFO(fmt(MSG_MAX_TRY,
+                      getCuid(),
+                      req_->getTryCount()));
+      A2_LOG_ERROR_EX(fmt(MSG_DOWNLOAD_ABORTED,
+                          getCuid(),
+                          req_->getUri().c_str()),
+                      err);
+      fileEntry_->addURIResult(req_->getUri(), err.getErrorCode());
+      requestGroup_->setLastErrorCode(err.getErrorCode());
+      if(err.getErrorCode() == error_code::CANNOT_RESUME) {
         requestGroup_->increaseResumeFailureCount();
       }
       onAbort();
@@ -307,10 +303,10 @@ bool AbstractCommand::execute() {
       return prepareForRetry(0);
     }
   } catch(DownloadFailureException& err) {
-    getLogger()->error(EX_EXCEPTION_CAUGHT, err);
-    if(!req_.isNull()) {
-      fileEntry_->addURIResult(req_->getUri(), err.getCode());
-      requestGroup_->setLastUriResult(req_->getUri(), err.getCode());
+    A2_LOG_ERROR_EX(EX_EXCEPTION_CAUGHT, err);
+    requestGroup_->setLastErrorCode(err.getErrorCode());
+    if(req_) {
+      fileEntry_->addURIResult(req_->getUri(), err.getErrorCode());
     }
     requestGroup_->setHaltRequested(true);
     return true;
@@ -325,19 +321,15 @@ void AbstractCommand::tryReserved() {
     // and there are no URI left. Because file length is unknown, we
     // can assume that there are no in-flight request object.
     if(entry->getLength() == 0 && entry->getRemainingUris().empty()) {
-      if(getLogger()->debug()) {
-        getLogger()->debug("CUID#%s - Not trying next request."
-                           " No reserved/pooled request is remaining and"
-                           " total length is still unknown.",
-                           util::itos(getCuid()).c_str());
-      }
+      A2_LOG_DEBUG(fmt("CUID#%lld - Not trying next request."
+                       " No reserved/pooled request is remaining and"
+                       " total length is still unknown.",
+                       getCuid()));
       return;
     }
   }
-  if(getLogger()->debug()) {
-    getLogger()->debug("CUID#%s - Trying reserved/pooled request.",
-                       util::itos(getCuid()).c_str());
-  }
+  A2_LOG_DEBUG(fmt("CUID#%lld - Trying reserved/pooled request.",
+                   getCuid()));
   std::vector<Command*> commands;
   requestGroup_->createNextCommand(commands, e_, 1);
   e_->setNoWait(true);
@@ -345,16 +337,14 @@ void AbstractCommand::tryReserved() {
 }
 
 bool AbstractCommand::prepareForRetry(time_t wait) {
-  if(!getPieceStorage().isNull()) {
+  if(getPieceStorage()) {
     getSegmentMan()->cancelSegment(getCuid());
   }
-  if(!req_.isNull()) {
+  if(req_) {
     fileEntry_->poolRequest(req_);
-    if(getLogger()->debug()) {
-      getLogger()->debug("CUID#%s - Pooling request URI=%s",
-                         util::itos(getCuid()).c_str(), req_->getUri().c_str());
-    }
-    if(!getSegmentMan().isNull()) {
+    A2_LOG_DEBUG(fmt("CUID#%lld - Pooling request URI=%s",
+                     getCuid(), req_->getUri().c_str()));
+    if(getSegmentMan()) {
       getSegmentMan()->recognizeSegmentFor(fileEntry_);
     }
   }
@@ -372,24 +362,19 @@ bool AbstractCommand::prepareForRetry(time_t wait) {
 }
 
 void AbstractCommand::onAbort() {
-  if(!req_.isNull()) {
-    // TODO This might be a problem if the failure is caused by proxy.
-    e_->getRequestGroupMan()->getOrCreateServerStat
-      (req_->getHost(), req_->getProtocol())->setError();
+  if(req_) {
     fileEntry_->removeIdenticalURI(req_->getUri());
     fileEntry_->removeRequest(req_);
   }
-  if(getLogger()->debug()) {
-    getLogger()->debug("CUID#%s - Aborting download",
-                       util::itos(getCuid()).c_str());
-  }
-  if(!getPieceStorage().isNull()) {
+  A2_LOG_DEBUG(fmt("CUID#%lld - Aborting download",
+                   getCuid()));
+  if(getPieceStorage()) {
     getSegmentMan()->cancelSegment(getCuid());
     // Don't do following process if BitTorrent is involved or files
     // in DownloadContext is more than 1. The latter condition is
     // limitation of current implementation.
     if(!getOption()->getAsBool(PREF_ALWAYS_RESUME) &&
-       !fileEntry_.isNull() &&
+       fileEntry_ &&
        getSegmentMan()->calculateSessionDownloadLength() == 0 &&
        !requestGroup_->p2pInvolved() &&
        getDownloadContext()->getFileEntries().size() == 1) {
@@ -399,19 +384,17 @@ void AbstractCommand::onAbort() {
         // Local file exists, but given servers(or at least contacted
         // ones) doesn't support resume. Let's restart download from
         // scratch.
-        getLogger()->notice("CUID#%s - Failed to resume download."
-                            " Download from scratch.",
-                            util::itos(getCuid()).c_str());
-        if(getLogger()->debug()) {
-          getLogger()->debug
-            ("CUID#%s - Gathering URIs that has CANNOT_RESUME error",
-             util::itos(getCuid()).c_str());
-        }
-        // Set PREF_ALWAYS_RESUME to V_TRUE to avoid repeating this
+        A2_LOG_NOTICE(fmt("CUID#%lld - Failed to resume download."
+                          " Download from scratch.",
+                          getCuid()));
+        A2_LOG_DEBUG(fmt("CUID#%lld - Gathering URIs that has CANNOT_RESUME"
+                         " error",
+                         getCuid()));
+        // Set PREF_ALWAYS_RESUME to A2_V_TRUE to avoid repeating this
         // process.
-        getOption()->put(PREF_ALWAYS_RESUME, V_TRUE);
+        getOption()->put(PREF_ALWAYS_RESUME, A2_V_TRUE);
         std::deque<URIResult> res;
-        fileEntry_->extractURIResult(res, downloadresultcode::CANNOT_RESUME);
+        fileEntry_->extractURIResult(res, error_code::CANNOT_RESUME);
         if(!res.empty()) {
           getSegmentMan()->cancelAllSegments();
           getSegmentMan()->eraseSegmentWrittenLengthMemo();
@@ -420,11 +403,9 @@ void AbstractCommand::onAbort() {
           uris.reserve(res.size());
           std::transform(res.begin(), res.end(), std::back_inserter(uris),
                          std::mem_fun_ref(&URIResult::getURI));
-          if(getLogger()->debug()) {
-            getLogger()->debug("CUID#%s - %lu URIs found.",
-                               util::itos(getCuid()).c_str(),
-                               static_cast<unsigned long int>(uris.size()));
-          }
+          A2_LOG_DEBUG(fmt("CUID#%lld - %lu URIs found.",
+                           getCuid(),
+                           static_cast<unsigned long int>(uris.size())));
           fileEntry_->addUris(uris.begin(), uris.end());
           getSegmentMan()->recognizeSegmentFor(fileEntry_);
         }
@@ -446,7 +427,7 @@ void AbstractCommand::setReadCheckSocket(const SocketHandle& socket) {
     disableReadCheckSocket();
   } else {
     if(checkSocketIsReadable_) {
-      if(readCheckTarget_ != socket) {
+      if(*readCheckTarget_ != *socket) {
         e_->deleteSocketForReadCheck(readCheckTarget_, this);
         e_->addSocketForReadCheck(socket, this);
         readCheckTarget_ = socket;
@@ -482,7 +463,7 @@ void AbstractCommand::setWriteCheckSocket(const SocketHandle& socket) {
     disableWriteCheckSocket();
   } else {
     if(checkSocketIsWritable_) {
-      if(writeCheckTarget_ != socket) {
+      if(*writeCheckTarget_ != *socket) {
         e_->deleteSocketForWriteCheck(writeCheckTarget_, this);
         e_->addSocketForWriteCheck(socket, this);
         writeCheckTarget_ = socket;
@@ -505,8 +486,9 @@ void AbstractCommand::setWriteCheckSocketIf
   }
 }
 
+namespace {
 // Returns proxy option value for the given protocol.
-static const std::string& getProxyOptionFor
+const std::string& getProxyOptionFor
 (const std::string& proxyPref, const SharedHandle<Option>& option)
 {
   if(option->defined(proxyPref)) {
@@ -515,10 +497,12 @@ static const std::string& getProxyOptionFor
     return option->get(PREF_ALL_PROXY);
   }
 }
+} // namespace
 
+namespace {
 // Returns proxy URI for given protocol.  If no proxy URI is defined,
 // then returns an empty string.
-static const std::string& getProxyUri
+const std::string& getProxyUri
 (const std::string& protocol, const SharedHandle<Option>& option)
 {
   if(protocol == Request::PROTO_HTTP) {
@@ -531,16 +515,21 @@ static const std::string& getProxyUri
     return A2STR::NIL;
   }
 }
+} // namespace
 
+namespace {
 // Returns true if proxy is defined for the given protocol. Otherwise
 // returns false.
-static bool isProxyRequest
+bool isProxyRequest
 (const std::string& protocol, const SharedHandle<Option>& option)
 {
   const std::string& proxyUri = getProxyUri(protocol, option);
-  return !proxyUri.empty() && Request().setUri(proxyUri);
+  uri::UriStruct us;
+  return !proxyUri.empty() && uri::parse(us, proxyUri);
 }
+} // namespace
 
+namespace {
 class DomainMatch {
 private:
   std::string hostname_;
@@ -556,9 +545,11 @@ public:
     }
   }
 };
+} // namespace
 
-static bool inNoProxy(const SharedHandle<Request>& req,
-                      const std::string& noProxy)
+namespace {
+bool inNoProxy(const SharedHandle<Request>& req,
+               const std::string& noProxy)
 {
   std::vector<std::string> entries;
   util::split(noProxy, std::back_inserter(entries), ",", true);
@@ -597,6 +588,7 @@ static bool inNoProxy(const SharedHandle<Request>& req,
   }
   return false;
 }
+} // namespace
 
 bool AbstractCommand::isProxyDefined() const
 {
@@ -614,15 +606,10 @@ SharedHandle<Request> AbstractCommand::createProxyRequest() const
   if(!proxy.empty()) {
     proxyRequest.reset(new Request());
     if(proxyRequest->setUri(proxy)) {
-      if(getLogger()->debug()) {
-        getLogger()->debug("CUID#%s - Using proxy",
-                           util::itos(getCuid()).c_str());
-      }
+      A2_LOG_DEBUG(fmt("CUID#%lld - Using proxy", getCuid()));
     } else {
-      if(getLogger()->debug()) {
-        getLogger()->debug("CUID#%s - Failed to parse proxy string",
-                           util::itos(getCuid()).c_str());
-      }
+      A2_LOG_DEBUG(fmt("CUID#%lld - Failed to parse proxy string",
+                       getCuid()));
       proxyRequest.reset();
     }
   }
@@ -633,16 +620,21 @@ SharedHandle<Request> AbstractCommand::createProxyRequest() const
 
 bool AbstractCommand::isAsyncNameResolverInitialized() const
 {
-  return !asyncNameResolver_.isNull();
+  return asyncNameResolver_;
 }
 
 void AbstractCommand::initAsyncNameResolver(const std::string& hostname)
 {
-  asyncNameResolver_.reset(new AsyncNameResolver());
-  if(getLogger()->info()) {
-    getLogger()->info(MSG_RESOLVING_HOSTNAME,
-                      util::itos(getCuid()).c_str(), hostname.c_str());
+  int family;
+  if(getOption()->getAsBool(PREF_ENABLE_ASYNC_DNS6)) {
+    family = AF_UNSPEC;
+  } else {
+    family = AF_INET;
   }
+  asyncNameResolver_.reset(new AsyncNameResolver(family));
+  A2_LOG_INFO(fmt(MSG_RESOLVING_HOSTNAME,
+                  getCuid(),
+                  hostname.c_str()));
   asyncNameResolver_->resolve(hostname);
   setNameResolverCheck(asyncNameResolver_);
 }
@@ -659,11 +651,12 @@ bool AbstractCommand::asyncResolveHostname()
       e_->getRequestGroupMan()->getOrCreateServerStat
         (req_->getHost(), req_->getProtocol())->setError();
     }
-    throw DL_ABORT_EX
-      (StringFormat(MSG_NAME_RESOLUTION_FAILED,
-                    util::itos(getCuid()).c_str(),
-                    asyncNameResolver_->getHostname().c_str(),
-                    asyncNameResolver_->getError().c_str()).str());
+    throw DL_ABORT_EX2
+      (fmt(MSG_NAME_RESOLUTION_FAILED,
+           getCuid(),
+           asyncNameResolver_->getHostname().c_str(),
+           asyncNameResolver_->getError().c_str()),
+       error_code::NAME_RESOLVE_ERROR);
   default:
     return false;
   }
@@ -676,7 +669,7 @@ const std::vector<std::string>& AbstractCommand::getResolvedAddresses()
 
 void AbstractCommand::setNameResolverCheck
 (const SharedHandle<AsyncNameResolver>& resolver) {
-  if(!resolver.isNull()) {
+  if(resolver) {
     nameResolverCheck_ = true;
     e_->addNameResolverCheck(resolver, this);
   }
@@ -684,7 +677,7 @@ void AbstractCommand::setNameResolverCheck
 
 void AbstractCommand::disableNameResolverCheck
 (const SharedHandle<AsyncNameResolver>& resolver) {
-  if(!resolver.isNull()) {
+  if(resolver) {
     nameResolverCheck_ = false;
     e_->deleteNameResolverCheck(resolver, this);
   }
@@ -701,6 +694,7 @@ std::string AbstractCommand::resolveHostname
 (std::vector<std::string>& addrs, const std::string& hostname, uint16_t port)
 {
   if(util::isNumericHost(hostname)) {
+    addrs.push_back(hostname);
     return hostname;
   }
   e_->findAllCachedIPAddresses(std::back_inserter(addrs), hostname, port);
@@ -726,12 +720,10 @@ std::string AbstractCommand::resolveHostname
         }
         res.resolve(addrs, hostname);
       }
-    if(getLogger()->info()) {
-      getLogger()->info(MSG_NAME_RESOLUTION_COMPLETE,
-                        util::itos(getCuid()).c_str(),
-                        hostname.c_str(),
-                        strjoin(addrs.begin(), addrs.end(), ", ").c_str());
-    }
+    A2_LOG_INFO(fmt(MSG_NAME_RESOLUTION_COMPLETE,
+                    getCuid(),
+                    hostname.c_str(),
+                    strjoin(addrs.begin(), addrs.end(), ", ").c_str()));
     for(std::vector<std::string>::const_iterator i = addrs.begin(),
           eoi = addrs.end(); i != eoi; ++i) {
       e_->cacheIPAddress(hostname, *i, port);
@@ -739,43 +731,20 @@ std::string AbstractCommand::resolveHostname
     ipaddr = e_->findCachedIPAddress(hostname, port);
   } else {
     ipaddr = addrs.front();
-    if(getLogger()->info()) {
-      getLogger()->info(MSG_DNS_CACHE_HIT,
-                        util::itos(getCuid()).c_str(), hostname.c_str(),
-                        strjoin(addrs.begin(), addrs.end(), ", ").c_str());
-    }
+    A2_LOG_INFO(fmt(MSG_DNS_CACHE_HIT,
+                    getCuid(),
+                    hostname.c_str(),
+                    strjoin(addrs.begin(), addrs.end(), ", ").c_str()));
   }
   return ipaddr;
 }
 
-// nextCommand is going to be managed by CheckIntegrityEntry which is
-// created in side this function. Don't release nextCommand after this
-// function call.
-void AbstractCommand::prepareForNextAction(Command* nextCommand)
+void AbstractCommand::prepareForNextAction
+(const SharedHandle<CheckIntegrityEntry>& checkEntry)
 {
-#ifdef ENABLE_MESSAGE_DIGEST
-  if(requestGroup_->downloadFinished() &&
-     getDownloadContext()->isChecksumVerificationNeeded()) {
-    if(getLogger()->info()) {
-      getLogger()->info(MSG_HASH_CHECK_NOT_DONE);
-    }
-    SharedHandle<CheckIntegrityEntry> entry
-      (new ChecksumCheckIntegrityEntry(requestGroup_));
-    if(entry->isValidationReady()) {
-      delete nextCommand;
-      entry->initValidator();
-      entry->cutTrailingGarbage();
-      e_->getCheckIntegrityMan()->pushEntry(entry);
-      return;
-    }
-  }
-#endif // ENABLE_MESSAGE_DIGEST
-  SharedHandle<CheckIntegrityEntry> entry
-    (new StreamCheckIntegrityEntry(requestGroup_, nextCommand));
-
   std::vector<Command*>* commands = new std::vector<Command*>();
   auto_delete_container<std::vector<Command*> > commandsDel(commands);
-  requestGroup_->processCheckIntegrityEntry(*commands, entry, e_);
+  requestGroup_->processCheckIntegrityEntry(*commands, checkEntry, e_);
   e_->addCommand(*commands);
   commands->clear();
   e_->setNoWait(true);
@@ -793,11 +762,9 @@ bool AbstractCommand::checkIfConnectionEstablished
       // See also InitiateConnectionCommand::executeInternal()
       e_->markBadIPAddress(connectedHostname, connectedAddr, connectedPort);
       if(!e_->findCachedIPAddress(connectedHostname, connectedPort).empty()) {
-        if(getLogger()->info()) {
-          getLogger()->info(MSG_CONNECT_FAILED_AND_RETRY,
-                            util::itos(getCuid()).c_str(),
-                            connectedAddr.c_str(), connectedPort);
-        }
+        A2_LOG_INFO(fmt(MSG_CONNECT_FAILED_AND_RETRY,
+                        getCuid(),
+                        connectedAddr.c_str(), connectedPort));
         Command* command =
           InitiateConnectionCommandFactory::createInitiateConnectionCommand
           (getCuid(), req_, fileEntry_, requestGroup_, e_);
@@ -813,7 +780,7 @@ bool AbstractCommand::checkIfConnectionEstablished
           (req_->getHost(), req_->getProtocol())->setError();
       }
       throw DL_RETRY_EX
-        (StringFormat(MSG_ESTABLISHING_CONNECTION_FAILED, error.c_str()).str());
+        (fmt(MSG_ESTABLISHING_CONNECTION_FAILED, error.c_str()));
     }
   }
   return true;
@@ -842,11 +809,41 @@ void AbstractCommand::createSocket()
 
 size_t AbstractCommand::calculateMinSplitSize() const
 {
-  if(!req_.isNull() && req_->isPipeliningEnabled()) {
+  if(req_ && req_->isPipeliningEnabled()) {
     return getDownloadContext()->getPieceLength();
   } else {
     return getOption()->getAsInt(PREF_MIN_SPLIT_SIZE);
   }
 }
 
+void AbstractCommand::setRequest(const SharedHandle<Request>& request)
+{
+  req_ = request;
+}
+
+void AbstractCommand::setFileEntry(const SharedHandle<FileEntry>& fileEntry)
+{
+  fileEntry_ = fileEntry;
+}
+
+void AbstractCommand::setSocket(const SharedHandle<SocketCore>& s)
+{
+  socket_ = s;
+}
+
+const SharedHandle<DownloadContext>& AbstractCommand::getDownloadContext() const
+{
+  return requestGroup_->getDownloadContext();
+}
+
+const SharedHandle<SegmentMan>& AbstractCommand::getSegmentMan() const
+{
+  return requestGroup_->getSegmentMan();
+}
+
+const SharedHandle<PieceStorage>& AbstractCommand::getPieceStorage() const
+{
+  return requestGroup_->getPieceStorage();
+}
+
 } // namespace aria2
