@@ -47,56 +47,64 @@
 #include "File.h"
 #include "bittorrent_helper.h"
 #include "DlAbortEx.h"
-#include "StringFormat.h"
+#include "fmt.h"
+#include "FileEntry.h"
 
 namespace aria2 {
 
-BtDependency::BtDependency(const WeakHandle<RequestGroup>& dependant,
-                           const SharedHandle<RequestGroup>& dependee):
-  _dependant(dependant),
-  _dependee(dependee),
-  _logger(LogFactory::getInstance()) {}
+BtDependency::BtDependency
+(RequestGroup* dependant,
+ const SharedHandle<RequestGroup>& dependee)
+  : dependant_(dependant),
+    dependee_(dependee)
+{}
 
 BtDependency::~BtDependency() {}
 
-static void copyValues(const SharedHandle<FileEntry>& d,
-                       const SharedHandle<FileEntry>& s)
+namespace {
+void copyValues(const SharedHandle<FileEntry>& d,
+                const SharedHandle<FileEntry>& s)
 {
   d->setRequested(true);
   d->setPath(s->getPath());
   d->addUris(s->getRemainingUris().begin(),
              s->getRemainingUris().end());
-  if(!s->isSingleHostMultiConnectionEnabled()) {
-    d->disableSingleHostMultiConnection();
-  }
+  d->setMaxConnectionPerServer(s->getMaxConnectionPerServer());
+  d->setUniqueProtocol(s->isUniqueProtocol());
 }
+} // namespace
 
 bool BtDependency::resolve()
 {
-  if(_dependee->getNumCommand() == 0 && _dependee->downloadFinished()) {
-    SharedHandle<RequestGroup> dependee = _dependee;
+  if(dependee_->getNumCommand() == 0 && dependee_->downloadFinished()) {
+    SharedHandle<RequestGroup> dependee = dependee_;
     // cut reference here
-    _dependee.reset();
+    dependee_.reset();
     SharedHandle<DownloadContext> context(new DownloadContext());
-    context->setDir(_dependant->getDownloadContext()->getDir());
     try {
       SharedHandle<DiskAdaptor> diskAdaptor =
         dependee->getPieceStorage()->getDiskAdaptor();
       diskAdaptor->openExistingFile();
       std::string content = util::toString(diskAdaptor);
       if(dependee->getDownloadContext()->hasAttribute(bittorrent::BITTORRENT)) {
-        const BDE& attrs =
-          dependee->getDownloadContext()->getAttribute(bittorrent::BITTORRENT);
+        SharedHandle<TorrentAttribute> attrs =
+          bittorrent::getTorrentAttrs(dependee->getDownloadContext());
         bittorrent::loadFromMemory
-          (bittorrent::metadata2Torrent(content, attrs), context, "default");
+          (bittorrent::metadata2Torrent(content, attrs), context,
+           dependant_->getOption(), "default");
+        // We don't call bittorrent::adjustAnnounceUri() because it
+        // has already been called with attrs.
       } else {
         bittorrent::loadFromMemory
-          (content, context, File(dependee->getFirstFilePath()).getBasename());
+          (content, context, dependant_->getOption(),
+           File(dependee->getFirstFilePath()).getBasename());
+        bittorrent::adjustAnnounceUri(bittorrent::getTorrentAttrs(context),
+                                      dependant_->getOption());
       }
       const std::vector<SharedHandle<FileEntry> >& fileEntries =
         context->getFileEntries();
       const std::vector<SharedHandle<FileEntry> >& dependantFileEntries =
-        _dependant->getDownloadContext()->getFileEntries();
+        dependant_->getDownloadContext()->getFileEntries();
       // If dependant's FileEntry::getOriginalName() is empty, we
       // assume that torrent is single file. In Metalink3, this is
       // always assumed.
@@ -106,9 +114,9 @@ bool BtDependency::resolve()
       } else {
         std::for_each(fileEntries.begin(), fileEntries.end(),
                       std::bind2nd(mem_fun_sh(&FileEntry::setRequested),false));
-        // Copy file path in _dependant's FileEntries to newly created
+        // Copy file path in dependant_'s FileEntries to newly created
         // context's FileEntries to endorse the path structure of
-        // _dependant.  URIs and singleHostMultiConnection are also copied.
+        // dependant_.  URIs and singleHostMultiConnection are also copied.
         std::vector<SharedHandle<FileEntry> >::const_iterator ctxFilesEnd =
           fileEntries.end();
         for(std::vector<SharedHandle<FileEntry> >::const_iterator s =
@@ -123,27 +131,28 @@ bool BtDependency::resolve()
           }
           if(d == ctxFilesEnd) {
             throw DL_ABORT_EX
-              (StringFormat("No entry %s in torrent file",
-                            (*s)->getOriginalName().c_str()).str());
+              (fmt("No entry %s in torrent file",
+                   (*s)->getOriginalName().c_str()));
           }
           copyValues(*d, *s);
         }
       }
     } catch(RecoverableException& e) {
-      _logger->error(EX_EXCEPTION_CAUGHT, e);
-      _logger->info("BtDependency for GID#%d failed. Go without Bt.",
-                    _dependant->getGID());
+      A2_LOG_ERROR_EX(EX_EXCEPTION_CAUGHT, e);
+      A2_LOG_INFO(fmt("BtDependency for GID#%s failed. Go without Bt.",
+                      util::itos(dependant_->getGID()).c_str()));
       return true;
     }
-    _logger->info("Dependency resolved for GID#%d", _dependant->getGID());
-    _dependant->setDownloadContext(context);
+    A2_LOG_INFO(fmt("Dependency resolved for GID#%s",
+                    util::itos(dependant_->getGID()).c_str()));
+    dependant_->setDownloadContext(context);
     return true;
-  } else if(_dependee->getNumCommand() == 0) {
-    // _dependee's download failed.
+  } else if(dependee_->getNumCommand() == 0) {
+    // dependee_'s download failed.
     // cut reference here
-    _dependee.reset();
-    _logger->info("BtDependency for GID#%d failed. Go without Bt.",
-                  _dependant->getGID());    
+    dependee_.reset();
+    A2_LOG_INFO(fmt("BtDependency for GID#%s failed. Go without Bt.",
+                    util::itos(dependant_->getGID()).c_str()));
     return true;
   } else {
     return false;
