@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_BT_REGISTRY_H_
-#define _D_BT_REGISTRY_H_
+#ifndef D_BT_REGISTRY_H
+#define D_BT_REGISTRY_H
 
 #include "common.h"
 
@@ -64,25 +64,17 @@ struct BtObject {
            const SharedHandle<PeerStorage>& peerStorage,
            const SharedHandle<BtAnnounce>& btAnnounce,
            const SharedHandle<BtRuntime>& btRuntime,
-           const SharedHandle<BtProgressInfoFile>& btProgressInfoFile):
-    downloadContext_(downloadContext),
-    pieceStorage_(pieceStorage),
-    peerStorage_(peerStorage),
-    btAnnounce_(btAnnounce),
-    btRuntime_(btRuntime),
-    btProgressInfoFile_(btProgressInfoFile) {}
+           const SharedHandle<BtProgressInfoFile>& btProgressInfoFile);
 
-  BtObject() {}
+  BtObject();
 
-  bool isNull() const
-  {
-    return downloadContext_.isNull() &&
-      pieceStorage_.isNull() &&
-      peerStorage_.isNull() &&
-      btAnnounce_.isNull() &&
-      btRuntime_.isNull() &&
-      btProgressInfoFile_.isNull();
-  }
+  BtObject(const BtObject& c);
+
+  ~BtObject();
+
+  BtObject& operator=(const BtObject& c);
+
+  bool isNull() const;
 };
 
 class BtRegistry {
@@ -116,4 +108,4 @@ public:
 
 } // namespace aria2
 
-#endif // _D_BT_REGISTRY_H_
+#endif // D_BT_REGISTRY_H
