@@ -45,7 +45,6 @@
 namespace aria2 {
 
 class Peer;
-class Logger;
 
 class BtSeederStateChoke {
 private:
@@ -53,8 +52,6 @@ private:
 
   Timer lastRound_;
 
-  Logger* logger_;
-
   class PeerEntry {
   private:
     SharedHandle<Peer> peer_;
@@ -66,6 +63,12 @@ private:
     const static time_t TIME_FRAME = 20;
   public:
     PeerEntry(const SharedHandle<Peer>& peer);
+    PeerEntry(const PeerEntry& c);
+    ~PeerEntry();
+
+    void swap(PeerEntry& c);
+
+    PeerEntry& operator=(const PeerEntry& c);
 
     bool operator<(const PeerEntry& rhs) const;
 
@@ -98,8 +101,14 @@ public:
   void executeChoke(const std::vector<SharedHandle<Peer> >& peerSet);
 
   const Timer& getLastRound() const { return lastRound_; }
+
+  friend void swap(PeerEntry& a, PeerEntry& b);
 };
 
+void swap
+(BtSeederStateChoke::PeerEntry& a,
+ BtSeederStateChoke::PeerEntry& b);
+
 } // namespace aria2
 
 #endif // D_BT_SEEDER_STATE_CHOKE_H
