@@ -39,13 +39,12 @@
 #include "HttpRequest.h"
 #include "Segment.h"
 #include "Socket.h"
-#include "DownloadContext.h"
 
 namespace aria2 {
 
 FtpTunnelResponseCommand::FtpTunnelResponseCommand
-(int cuid,
- const RequestHandle& req,
+(cuid_t cuid,
+ const SharedHandle<Request>& req,
  const SharedHandle<FileEntry>& fileEntry,
  RequestGroup* requestGroup,
  const HttpConnectionHandle& httpConnection,
@@ -59,8 +58,9 @@ FtpTunnelResponseCommand::~FtpTunnelResponseCommand() {}
 
 Command* FtpTunnelResponseCommand::getNextCommand()
 {
-  return new FtpNegotiationCommand(cuid, req, _fileEntry,
-                                   _requestGroup, e, socket);
+  return new FtpNegotiationCommand(getCuid(), getRequest(), getFileEntry(),
+                                   getRequestGroup(), getDownloadEngine(),
+                                   getSocket());
 }
 
 } // namespace aria2
