@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -46,17 +46,21 @@
 #include "Logger.h"
 #include "Base64.h"
 #include "a2functional.h"
+#include "fmt.h"
 
 namespace aria2 {
 
-HttpServer::HttpServer(const SharedHandle<SocketCore>& socket,
-		       DownloadEngine* e):
-  _socket(socket),
-  _socketBuffer(socket),
-  _e(e),
-  _headerProcessor(new HttpHeaderProcessor()),
-  _logger(LogFactory::getInstance()),
-  _keepAlive(true)
+HttpServer::HttpServer
+(const SharedHandle<SocketCore>& socket,
+ DownloadEngine* e)
+ : socket_(socket),
+   socketBuffer_(socket),
+   e_(e),
+   headerProcessor_(new HttpHeaderProcessor()),
+   keepAlive_(true),
+   gzip_(false),
+   acceptsPersistentConnection_(true),
+   acceptsGZip_(false)
 {}
 
 HttpServer::~HttpServer() {}
@@ -65,74 +69,73 @@ SharedHandle<HttpHeader> HttpServer::receiveRequest()
 {
   size_t size = 512;
   unsigned char buf[size];
-  _socket->peekData(buf, size);
-  if(size == 0 && !(_socket->wantRead() || _socket->wantWrite())) {
+  socket_->peekData(buf, size);
+  if(size == 0 && !(socket_->wantRead() || socket_->wantWrite())) {
     throw DL_ABORT_EX(EX_EOF_FROM_PEER);
   }
-  _headerProcessor->update(buf, size);
-  if(!_headerProcessor->eoh()) {
-    _socket->readData(buf, size);
+  headerProcessor_->update(buf, size);
+  if(!headerProcessor_->eoh()) {
+    socket_->readData(buf, size);
     return SharedHandle<HttpHeader>();
   }
-  size_t putbackDataLength = _headerProcessor->getPutBackDataLength();
+  size_t putbackDataLength = headerProcessor_->getPutBackDataLength();
   size -= putbackDataLength;
-  _socket->readData(buf, size);
+  socket_->readData(buf, size);
 
-  SharedHandle<HttpHeader> header = _headerProcessor->getHttpRequestHeader();
-  if(!header.isNull()) {
-    _logger->info("HTTP Server received request\n%s",
-		  _headerProcessor->getHeaderString().c_str());
-    _lastRequestHeader = header;
-    _lastBody.clear();
-    _lastBody.str("");
-    _lastContentLength =
-      _lastRequestHeader->getFirstAsUInt(HttpHeader::CONTENT_LENGTH);
-    _headerProcessor->clear();
+  SharedHandle<HttpHeader> header = headerProcessor_->getHttpRequestHeader();
+  if(header) {
+    A2_LOG_INFO(fmt("HTTP Server received request\n%s",
+                    headerProcessor_->getHeaderString().c_str()));
+    lastRequestHeader_ = header;
+    lastBody_.clear();
+    lastBody_.str("");
+    lastContentLength_ =
+      lastRequestHeader_->getFirstAsUInt(HttpHeader::CONTENT_LENGTH);
+    headerProcessor_->clear();
+
+    std::string connection =
+      util::toLower(lastRequestHeader_->getFirst(HttpHeader::CONNECTION));
+    acceptsPersistentConnection_ =
+      connection.find(HttpHeader::CLOSE) == std::string::npos &&
+      (lastRequestHeader_->getVersion() == HttpHeader::HTTP_1_1 ||
+       connection.find("keep-alive") != std::string::npos);
+
+    std::vector<std::string> acceptEncodings;
+    util::split(lastRequestHeader_->getFirst(HttpHeader::ACCEPT_ENCODING),
+                std::back_inserter(acceptEncodings), A2STR::COMMA_C, true);
+    acceptsGZip_ =
+      std::find(acceptEncodings.begin(), acceptEncodings.end(), "gzip")
+      != acceptEncodings.end();
   }
-
   return header;
 }
 
 bool HttpServer::receiveBody()
 {
-  if(_lastContentLength == 0) {
+  if(lastContentLength_ == 0) {
     return true;
   }
   const size_t BUFLEN = 4096;
   char buf[BUFLEN];
   size_t length = std::min(BUFLEN,
-			   static_cast<size_t>
-			   (_lastContentLength-_lastBody.tellg()));
-  _socket->readData(buf, length);
-  if(length == 0 && !(_socket->wantRead() || _socket->wantWrite())) {
+                           static_cast<size_t>
+                           (lastContentLength_-lastBody_.tellg()));
+  socket_->readData(buf, length);
+  if(length == 0 && !(socket_->wantRead() || socket_->wantWrite())) {
     throw DL_ABORT_EX(EX_EOF_FROM_PEER);
   }
-  _lastBody.write(buf, length);
-  return _lastContentLength == static_cast<uint64_t>(_lastBody.tellp());
+  lastBody_.write(buf, length);
+  return lastContentLength_ == static_cast<uint64_t>(lastBody_.tellp());
 }
 
 std::string HttpServer::getBody() const
 {
-  return _lastBody.str();
+  return lastBody_.str();
 }
 
 const std::string& HttpServer::getRequestPath() const
 {
-  return _lastRequestHeader->getRequestPath();
-}
-
-bool HttpServer::supportsPersistentConnection() const
-{
-  if(!_keepAlive) {
-    return false;
-  }
-
-  std::string connection =
-    util::toLower(_lastRequestHeader->getFirst(HttpHeader::CONNECTION));
-
-  return connection.find(HttpHeader::CLOSE) == std::string::npos &&
-    (_lastRequestHeader->getVersion() == HttpHeader::HTTP_1_1 ||
-     connection.find("keep-alive") != std::string::npos);
+  return lastRequestHeader_->getRequestPath();
 }
 
 void HttpServer::feedResponse(const std::string& text, const std::string& contentType)
@@ -141,15 +144,17 @@ void HttpServer::feedResponse(const std::string& text, const std::string& conten
 }
 
 void HttpServer::feedResponse(const std::string& status,
-			      const std::string& headers,
-			      const std::string& text,
-			      const std::string& contentType)
+                              const std::string& headers,
+                              const std::string& text,
+                              const std::string& contentType)
 {
   std::string header = "HTTP/1.1 ";
   strappend(header, status, "\r\n",
-	    "Content-Type: ", contentType, "\r\n",
-	    "Content-Length: ", util::uitos(text.size()), "\r\n");
-
+            "Content-Type: ", contentType, "\r\n",
+            "Content-Length: ", util::uitos(text.size()), "\r\n");
+  if(supportsGZip()) {
+    header += "Content-Encoding: gzip\r\n";
+  }
   if(!supportsPersistentConnection()) {
     header += "Connection: close\r\n";
   }
@@ -161,40 +166,47 @@ void HttpServer::feedResponse(const std::string& status,
   }
 
   header += "\r\n";
-
-  _logger->debug("HTTP Server sends response:\n%s", header.c_str());
-		 
-  _socketBuffer.feedSendBuffer(header);
-  _socketBuffer.feedSendBuffer(text);
+  A2_LOG_DEBUG(fmt("HTTP Server sends response:\n%s", header.c_str()));
+  socketBuffer_.pushStr(header);
+  socketBuffer_.pushStr(text);
 }
 
 ssize_t HttpServer::sendResponse()
 {
-  return _socketBuffer.send();
+  return socketBuffer_.send();
 }
 
 bool HttpServer::sendBufferIsEmpty() const
 {
-  return _socketBuffer.sendBufferIsEmpty();
+  return socketBuffer_.sendBufferIsEmpty();
 }
 
 bool HttpServer::authenticate()
 {
-  if(_username.empty()) {
+  if(username_.empty()) {
     return true;
   }
 
-  std::string authHeader = _lastRequestHeader->getFirst("Authorization");
+  std::string authHeader = lastRequestHeader_->getFirst("Authorization");
   if(authHeader.empty()) {
     return false;
   }
-  std::pair<std::string, std::string> p = util::split(authHeader, " ");
+  std::pair<std::string, std::string> p;
+  util::divide(p, authHeader, ' ');
   if(p.first != "Basic") {
     return false;
   }
   std::string userpass = Base64::decode(p.second);
-  std::pair<std::string, std::string> userpassPair = util::split(userpass, ":");
-  return _username == userpassPair.first && _password == userpassPair.second;
+  std::pair<std::string, std::string> userpassPair;
+  util::divide(userpassPair, userpass, ':');
+  return username_ == userpassPair.first && password_ == userpassPair.second;
+}
+
+void HttpServer::setUsernamePassword
+(const std::string& username, const std::string& password)
+{
+  username_ = username;
+  password_ = password;
 }
 
 } // namespace aria2
