@@ -32,49 +32,58 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_REQUEST_GROUP_MAN_H_
-#define _D_REQUEST_GROUP_MAN_H_
+#ifndef D_REQUEST_GROUP_MAN_H
+#define D_REQUEST_GROUP_MAN_H
 
 #include "common.h"
 
 #include <string>
 #include <deque>
 #include <iosfwd>
+#include <vector>
 
 #include "SharedHandle.h"
 #include "DownloadResult.h"
 #include "TransferStat.h"
+#include "RequestGroup.h"
 
 namespace aria2 {
 
 class DownloadEngine;
-class RequestGroup;
 class Command;
-class Logger;
-class DownloadResult;
+struct DownloadResult;
 class ServerStatMan;
 class ServerStat;
 class Option;
 
 class RequestGroupMan {
 private:
-  std::deque<SharedHandle<RequestGroup> > _requestGroups;
-  std::deque<SharedHandle<RequestGroup> > _reservedGroups;
-  std::deque<SharedHandle<DownloadResult> > _downloadResults;
-  Logger* _logger;
-  unsigned int _maxSimultaneousDownloads;
-  int32_t _gidCounter;
+  std::deque<SharedHandle<RequestGroup> > requestGroups_;
+  std::deque<SharedHandle<RequestGroup> > reservedGroups_;
+  std::deque<SharedHandle<DownloadResult> > downloadResults_;
+  unsigned int maxSimultaneousDownloads_;
 
-  const Option* _option;
+  const Option* option_;
 
-  SharedHandle<ServerStatMan> _serverStatMan;
+  SharedHandle<ServerStatMan> serverStatMan_;
 
-  unsigned int _maxOverallDownloadSpeedLimit;
+  unsigned int maxOverallDownloadSpeedLimit_;
 
-  unsigned int _maxOverallUploadSpeedLimit;
+  unsigned int maxOverallUploadSpeedLimit_;
 
   // truf if XML-RPC is enabled.
-  bool _xmlRpc;
+  bool xmlRpc_;
+
+  bool queueCheck_;
+
+  // The number of error DownloadResult removed because of upper limit
+  // of the queue
+  size_t removedErrorResult_;
+
+  // The last error of removed DownloadResult
+  error_code::Value removedLastErrorResult_;
+
+  size_t maxDownloadResult_;
 
   std::string
   formatDownloadResult(const std::string& status,
@@ -83,10 +92,12 @@ private:
   void configureRequestGroup
   (const SharedHandle<RequestGroup>& requestGroup) const;
 public:
-  RequestGroupMan(const std::deque<SharedHandle<RequestGroup> >& requestGroups,
+  RequestGroupMan(const std::vector<SharedHandle<RequestGroup> >& requestGroups,
                   unsigned int maxSimultaneousDownloads,
                   const Option* option);
 
+  ~RequestGroupMan();
+
   bool downloadFinished();
 
   void save();
@@ -103,12 +114,12 @@ public:
 
   void addRequestGroup(const SharedHandle<RequestGroup>& group);
 
-  void addReservedGroup(const std::deque<SharedHandle<RequestGroup> >& groups);
+  void addReservedGroup(const std::vector<SharedHandle<RequestGroup> >& groups);
 
   void addReservedGroup(const SharedHandle<RequestGroup>& group);
 
   void insertReservedGroup
-  (size_t pos, const std::deque<SharedHandle<RequestGroup> >& groups);
+  (size_t pos, const std::vector<SharedHandle<RequestGroup> >& groups);
 
   void insertReservedGroup(size_t pos, const SharedHandle<RequestGroup>& group);
 
@@ -118,17 +129,17 @@ public:
   
   const std::deque<SharedHandle<RequestGroup> >& getRequestGroups() const
   {
-    return _requestGroups;
+    return requestGroups_;
   }
 
-  SharedHandle<RequestGroup> findRequestGroup(int32_t gid) const;
+  SharedHandle<RequestGroup> findRequestGroup(gid_t gid) const;
 
   const std::deque<SharedHandle<RequestGroup> >& getReservedGroups() const
   {
-    return _reservedGroups;
+    return reservedGroups_;
   }
 
-  SharedHandle<RequestGroup> findReservedGroup(int32_t gid) const;
+  SharedHandle<RequestGroup> findReservedGroup(gid_t gid) const;
 
   enum HOW {
     POS_SET,
@@ -145,9 +156,9 @@ public:
   // beyond the end of the queue, it moves the download to the
   // beginning or the end of the queue respectively.  Returns the
   // destination position.
-  size_t changeReservedGroupPosition(int32_t gid, int pos, HOW how);
+  size_t changeReservedGroupPosition(gid_t gid, int pos, HOW how);
 
-  bool removeReservedGroup(int32_t gid);
+  bool removeReservedGroup(gid_t gid);
 
   void showDownloadResults(std::ostream& o) const;
 
@@ -157,37 +168,37 @@ public:
 
   class DownloadStat {
   private:
-    size_t _completed;
-    size_t _error;
-    size_t _inProgress;
-    size_t _waiting;
-    downloadresultcode::RESULT _lastErrorResult;
+    size_t completed_;
+    size_t error_;
+    size_t inProgress_;
+    size_t waiting_;
+    error_code::Value lastErrorResult_;
   public:
     DownloadStat(size_t completed,
                  size_t error,
                  size_t inProgress,
                  size_t waiting,
-                 downloadresultcode::RESULT lastErrorResult =
-                 downloadresultcode::FINISHED):
-      _completed(completed),
-      _error(error),
-      _inProgress(inProgress),
-      _waiting(waiting),
-      _lastErrorResult(lastErrorResult) {}
+                 error_code::Value lastErrorResult =
+                 error_code::FINISHED):
+      completed_(completed),
+      error_(error),
+      inProgress_(inProgress),
+      waiting_(waiting),
+      lastErrorResult_(lastErrorResult) {}
 
-    downloadresultcode::RESULT getLastErrorResult() const
+    error_code::Value getLastErrorResult() const
     {
-      return _lastErrorResult;
+      return lastErrorResult_;
     }
 
     bool allCompleted() const
     {
-      return _error == 0 && _inProgress == 0 && _waiting == 0;
+      return error_ == 0 && inProgress_ == 0 && waiting_ == 0;
     }
 
     size_t getInProgress() const
     {
-      return _inProgress;
+      return inProgress_;
     }
   };
 
@@ -195,14 +206,16 @@ public:
 
   const std::deque<SharedHandle<DownloadResult> >& getDownloadResults() const
   {
-    return _downloadResults;
+    return downloadResults_;
   }
 
-  SharedHandle<DownloadResult> findDownloadResult(int32_t gid) const;
+  SharedHandle<DownloadResult> findDownloadResult(gid_t gid) const;
 
   // Removes all download results.
   void purgeDownloadResult();
 
+  void addDownloadResult(const SharedHandle<DownloadResult>& downloadResult);
+
   SharedHandle<ServerStat> findServerStat(const std::string& hostname,
                                           const std::string& protocol) const;
 
@@ -220,43 +233,64 @@ public:
   void removeStaleServerStat(time_t timeout);
 
   // Returns true if current download speed exceeds
-  // _maxOverallDownloadSpeedLimit.  Always returns false if
-  // _maxOverallDownloadSpeedLimit == 0.  Otherwise returns false.
+  // maxOverallDownloadSpeedLimit_.  Always returns false if
+  // maxOverallDownloadSpeedLimit_ == 0.  Otherwise returns false.
   bool doesOverallDownloadSpeedExceed();
 
   void setMaxOverallDownloadSpeedLimit(unsigned int speed)
   {
-    _maxOverallDownloadSpeedLimit = speed;
+    maxOverallDownloadSpeedLimit_ = speed;
   }
 
   unsigned int getMaxOverallDownloadSpeedLimit() const
   {
-    return _maxOverallDownloadSpeedLimit;
+    return maxOverallDownloadSpeedLimit_;
   }
 
   // Returns true if current upload speed exceeds
-  // _maxOverallUploadSpeedLimit. Always returns false if
-  // _maxOverallUploadSpeedLimit == 0. Otherwise returns false.
+  // maxOverallUploadSpeedLimit_. Always returns false if
+  // maxOverallUploadSpeedLimit_ == 0. Otherwise returns false.
   bool doesOverallUploadSpeedExceed();
 
   void setMaxOverallUploadSpeedLimit(unsigned int speed)
   {
-    _maxOverallUploadSpeedLimit = speed;
+    maxOverallUploadSpeedLimit_ = speed;
   }
 
   unsigned int getMaxOverallUploadSpeedLimit() const
   {
-    return _maxOverallUploadSpeedLimit;
+    return maxOverallUploadSpeedLimit_;
   }
 
   void setMaxSimultaneousDownloads(unsigned int max)
   {
-    _maxSimultaneousDownloads = max;
+    maxSimultaneousDownloads_ = max;
   }
+
+  // Call this function if requestGroups_ queue should be maintained.
+  // This function is added to reduce the call of maintenance, but at
+  // the same time, it provides fast maintenance reaction.
+  void requestQueueCheck()
+  {
+    queueCheck_ = true;
+  }
+
+  void clearQueueCheck()
+  {
+    queueCheck_ = false;
+  }
+
+  bool queueCheckRequested() const
+  {
+    return queueCheck_;
+  }
+
+  // Returns currently used hosts and its use count.
+  void getUsedHosts(std::vector<std::pair<size_t, std::string> >& usedHosts);
 };
 
 typedef SharedHandle<RequestGroupMan> RequestGroupManHandle;
 
 } // namespace aria2
 
-#endif // _D_REQUEST_GROUP_MAN_H_
+#endif // D_REQUEST_GROUP_MAN_H
