@@ -39,8 +39,8 @@ namespace aria2 {
 /**
  * Constants
  */
-const std::string V_TRUE("true");
-const std::string V_FALSE("false");
+const std::string A2_V_TRUE("true");
+const std::string A2_V_FALSE("false");
 const std::string V_NONE("none");
 const std::string V_MEM("mem");
 const std::string V_ALL("all");
@@ -198,6 +198,10 @@ const std::string PREF_MIN_SPLIT_SIZE("min-split-size");
 const std::string PREF_CONDITIONAL_GET("conditional-get");
 // value: true | false
 const std::string PREF_SELECT_LEAST_USED_HOST("select-least-used-host");
+// value: true | false
+const std::string PREF_ENABLE_ASYNC_DNS6("enable-async-dns6");
+// value: 1*digit
+const std::string PREF_MAX_DOWNLOAD_RESULT("max-download-result");
 
 /**
  * FTP related preferences
@@ -306,6 +310,8 @@ const std::string PREF_PEER_ID_PREFIX("peer-id-prefix");
 const std::string PREF_ENABLE_PEER_EXCHANGE("enable-peer-exchange");
 // values: true | false
 const std::string PREF_ENABLE_DHT("enable-dht");
+// values: a string
+const std::string PREF_DHT_LISTEN_ADDR("dht-listen-addr");
 // values: 1*digit
 const std::string PREF_DHT_LISTEN_PORT("dht-listen-port");
 // values: a string
@@ -316,6 +322,18 @@ const std::string PREF_DHT_ENTRY_POINT_PORT("dht-entry-point-port");
 const std::string PREF_DHT_ENTRY_POINT("dht-entry-point");
 // values: a string
 const std::string PREF_DHT_FILE_PATH("dht-file-path");
+// values: true | false
+const std::string PREF_ENABLE_DHT6("enable-dht6");
+// values: a string
+const std::string PREF_DHT_LISTEN_ADDR6("dht-listen-addr6");
+// values: a string
+const std::string PREF_DHT_ENTRY_POINT_HOST6("dht-entry-point-host6");
+// values: 1*digit
+const std::string PREF_DHT_ENTRY_POINT_PORT6("dht-entry-point-port6");
+// values: a string (hostname:port)
+const std::string PREF_DHT_ENTRY_POINT6("dht-entry-point6");
+// values: a string
+const std::string PREF_DHT_FILE_PATH6("dht-file-path6");
 // values: plain | arc4
 const std::string PREF_BT_MIN_CRYPTO_LEVEL("bt-min-crypto-level");
 const std::string V_PLAIN("plain");
@@ -358,6 +376,10 @@ const std::string PREF_BT_TRACKER_CONNECT_TIMEOUT("bt-tracker-connect-timeout");
 const std::string PREF_DHT_MESSAGE_TIMEOUT("dht-message-timeout");
 // values: string
 const std::string PREF_ON_BT_DOWNLOAD_COMPLETE("on-bt-download-complete");
+// values: string
+const std::string PREF_BT_TRACKER("bt-tracker");
+// values: string
+const std::string PREF_BT_EXCLUDE_TRACKER("bt-exclude-tracker");
 
 /**
  * Metalink related preferences
